@@ -162,7 +162,6 @@ mapi_settings_run_folder_size_dialog (const gchar *profile, gpointer data)
 	GtkWidget *spinner, *alignment;
 	GtkWidget *spinner_label;
 	FolderSizeDialogData *dialog_data;
-	GThread *folder_list_thread;
 
 	dialog_data = g_new0 (FolderSizeDialogData, 1);
 
@@ -195,7 +194,7 @@ mapi_settings_run_folder_size_dialog (const gchar *profile, gpointer data)
 	dialog_data->profile = g_strdup (profile);
 
 	/* Fetch folder list and size information in a thread */
-	folder_list_thread = g_thread_create (mapi_settings_get_folder_size, dialog_data, TRUE, NULL);
+	g_thread_create (mapi_settings_get_folder_size, dialog_data, FALSE, NULL);
 
 	/* Start the dialog */
 	gtk_dialog_run (dialog_data->dialog);
@@ -259,9 +258,12 @@ action_folder_size_cb (GtkAction *action,
 
 			if (store && CAMEL_IS_SERVICE (store)) {
 				CamelService *service = CAMEL_SERVICE (store);
+				CamelURL *url;
 
-				if (service->url)
-					profile = g_strdup (camel_url_get_param (service->url, "profile"));
+				url = camel_service_get_camel_url (service);
+
+				if (url)
+					profile = g_strdup (camel_url_get_param (url, "profile"));
 			}
 		}
 	}
@@ -279,7 +281,6 @@ action_folder_size_cb (GtkAction *action,
 static void
 folder_size_actions_update_cb (EShellView *shell_view, GtkActionEntry *entries)
 {
-	EShellContent *shell_content;
 	EShellWindow *shell_window;
 	GtkActionGroup *action_group;
 	GtkUIManager *ui_manager;
@@ -300,7 +301,6 @@ folder_size_actions_update_cb (EShellView *shell_view, GtkActionEntry *entries)
 		return;
 	}
 
-	shell_content = e_shell_view_get_shell_content (shell_view);
 	shell_window = e_shell_view_get_shell_window (shell_view);
 
 	ui_manager = e_shell_window_get_ui_manager (shell_window);
@@ -340,7 +340,7 @@ org_gnome_exchange_mapi_settings (EPlugin *epl, EConfigHookItemFactoryData *data
 
 	target_account = (EMConfigTargetAccount *)data->config->target;
 
-	source_url = e_account_get_string (target_account->account,  E_ACCOUNT_SOURCE_URL);
+	source_url = e_account_get_string (target_account->modified_account,  E_ACCOUNT_SOURCE_URL);
 
 	url = camel_url_new(source_url, NULL);
 	if (url == NULL || strcmp(url->protocol, "mapi") != 0) {
@@ -369,7 +369,7 @@ org_gnome_exchange_mapi_settings (EPlugin *epl, EConfigHookItemFactoryData *data
 					      _("View the size of all Exchange folders"), NULL);
 	gtk_misc_set_alignment (GTK_MISC (lbl_fsize), 0, 0.5);
 	btn_fsize = (GtkButton*) g_object_new (GTK_TYPE_BUTTON, "label", _("Folder Size"), NULL);
-	g_signal_connect (btn_fsize, "clicked", G_CALLBACK (folder_size_clicked), target_account->account);
+	g_signal_connect (btn_fsize, "clicked", G_CALLBACK (folder_size_clicked), target_account->modified_account);
 	gtk_table_attach_defaults (tbl_misc, GTK_WIDGET (lbl_fsize), 0, 1, 0, 1);
 	gtk_table_attach (tbl_misc, GTK_WIDGET (btn_fsize), 1, 2, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
 
