@@ -42,7 +42,6 @@
 #include "camel-mapi-store.h"
 #include "camel-mapi-folder.h"
 #include "camel-mapi-store-summary.h"
-#include "camel-mapi-utils.h"
 #define d(x)
 
 #include <exchange-mapi-defs.h>
@@ -74,7 +73,7 @@ mapi_message_item_send (ExchangeMapiConnection *conn, MailItem *item, GError **p
 	#undef unset
 
 	mid = exchange_mapi_connection_create_item (conn, olFolderSentMail, fid,
-					 camel_mapi_utils_create_item_build_props, item,
+					 mapi_mail_utils_create_item_build_props, item,
 					 item->recipients, item->attachments, item->generic_streams, MAPI_OPTIONS_DELETE_ON_SUBMIT_FAILURE, perror);
 
 	return mid;
@@ -102,7 +101,7 @@ mapi_send_to_sync (CamelTransport *transport,
 
 	g_return_val_if_fail (CAMEL_IS_SERVICE (transport), FALSE);
 
-	url = CAMEL_SERVICE (transport)->url;
+	url = camel_service_get_camel_url (CAMEL_SERVICE (transport));
 	g_return_val_if_fail (url != NULL, FALSE);
 
 	conn = exchange_mapi_connection_find (camel_url_get_param (url, "profile"));
@@ -115,7 +114,7 @@ mapi_send_to_sync (CamelTransport *transport,
 	}
 
 	/* Convert MIME to MailItem, attacment lists and recipient list.*/
-	item = camel_mapi_utils_mime_to_item (message, 0, from, cancellable, NULL);
+	item = mapi_mime_message_to_mail_item (message, 0, from, cancellable, NULL);
 
 	/* send */
 	st = mapi_message_item_send (conn, item, error);
@@ -142,13 +141,17 @@ mapi_send_to_sync (CamelTransport *transport,
 static gchar *
 mapi_transport_get_name(CamelService *service, gboolean brief)
 {
+	CamelURL *url;
+
+	url = camel_service_get_camel_url (service);
+
 	if (brief) {
 		/* Translators: The %s is replaced with a server's host name */
-		return g_strdup_printf (_("Exchange MAPI server %s"), service->url->host);
+		return g_strdup_printf (_("Exchange MAPI server %s"), url->host);
 	} else {
 		/* Translators: The first %s is replaced with a user name, the second with a server's host name */
 		return g_strdup_printf (_("Exchange MAPI service for %s on %s"),
-					service->url->user, service->url->host);
+					url->user, url->host);
 	}
 }
 
